% Ten Tusscher et al model
% Am J Physiol 286: H1573H1589, 2004.
% human ventricular cell 
% improved version of Priebe-Beucklemann
%    
%    t   time variable
%    V   membrane potantial
clear all
load tt_fixed_params
colors = repmat('krgbmc',1,500) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 1:  Define all constants
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global F R T RTF ;
% Physical constants
F = 96.4867 ;                % Faraday constant, coulombs/mmol
R = 8.314 ;                  % gas constant, J/(K mol)
T = 273+37 ;                 % absolute temperature, K 
RTF = R*T/F ;

global Cm S ;
Cm = 1 ;                      % uF/cm2
S = 0.2 ;                     % surface area to volume ratio

global Vmyo VSR ;
Vmyo = 16.404 ;               % pL
VSR = 1.094 ;                 % pL

global Ko Nao Cao ;
Ko = 5400 ;                   % uM
Nao = 140000 ;                % uM
Cao = 2000 ;                  % uM

global GNa_ PCa_ GK1_ GKr_ GKs_ Gto_ ;
global GpK_ GpCa_ GNab_ GCab_ pKNa ;
GNa_ = 14.838 ;               % nS/pF
PCa_ = 1.75e-4 ;              % cm3 uF-1 s-1 
GK1_ = 5.405 ;                % nS/pF
GKr_ = 0.096 ;                % nS/pF

GpK_ = 0.0146 ;               % nS/pF
GpCa_ = 0.025 ;               % nS/pF
GNab_ = 2.9e-4 ;              % nS/pF
GCab_ = 5.92e-4 ;             % nS/pF

% % % Epicardial cell
Gto_ = 0.294 ;                % nS/pF
GKs_ = 0.245 ;                % nS/pF

% % % % M cell
% Gto_ = 0.294 ;                % nS/pF
% GKs_ = 0.062 ;                % nS/pF
% 
% % % % Endocardial cell
% Gto_ = 0.073 ;                % nS/pF
% GKs_ = 0.245 ;                % nS/pF
% 

% % % % Epicardial cell, M cell
% Gto_ = 0.294 ;                 % nS/pF
% % % % Endocardial cell
% Gto_ = 0.073 ;                 % nS/pF
% 
% % % % Epicardial cell, endocardial cell
% GKs_ = 0.245 ;                 % nS/pF
% % % % M cell
% GKs_ = 0.062 ;                 % nS/pF

pKNa = 0.03 ;                  % relative permeability, Na to K

global INaK_ kNaCa Iup_ Vleak
% Maximum rates of intracellular transport mechanisms
INaK_ = 1.362 ;                % pA/pF
kNaCa = 1000 ;                 % pA/pF
Iup_ = 0.425 ;                 % uM/ms
Vleak = 8e-5 ;                 % ms-1

global KmNa KmCa ksat alpha_ncx eta KmNai KmKo KpCa Kmup ;
% global eta ksat KmNa KmCa KmfCa KmK1 KmNai KmKo KmpCa Kmup  
% global KmNa KmCa KmNai KmKo sigma ;
% % values needed for calculation of NCX current
% Several saturation constants, nomenclature gets tricky
% NCX
KmNa = 87500 ;                 % uM
KmCa = 1380 ;                  % uM
ksat = 0.1 ;                   % unitless
alpha_ncx = 2.5 ;              % unitless
eta = 0.35 ;                   % unitless, actually gamma in paper
% Na-K pump
KmNai = 40000 ;                % uM
KmKo = 1000 ;                  % uM
% Sarcolemmal Ca pump
KpCa = 0.5 ;                    % uM
% SERCA
Kmup = 0.25 ;                  % uM

global arel brel crel ;
arel = 16.464 ;                 % uM/ms
brel = 250 ;                   % uM
crel = 8.232 ;                  % uM/ms

% % % Buffering parameters
% % % Considered generic cytotolic and SR buffers
global Bufc Kbufc BufSR KbufSR 
Bufc = 150 ;                % uM
Kbufc = 1 ;                   % uM
BufSR = 10000 ;              % uM
KbufSR = 300 ;                 % uM 

baselineparameters = [GNa_;GNab_;PCa_;GCab_;Gto_;GKr_;GKs_;GK1_;GpK_; ...
  INaK_;kNaCa;arel;crel;Vleak;Iup_;GpCa_] ;

n_parameters = length(baselineparameters) ;
variations = 300 ;
% variations = 1 ;
sigma = 0.1 ;     % standard deviation of variation of parameters
% sigma = 0 ;

 stimint_initial = 500 ; 
 
%  stimints = 370:-5:100 ;
%  %stimints = 400 ;
%  pacingrates = length(stimints) ;
%  allparameters = zeros(n_parameters,variations) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 2:  Define simulation and stimulus parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

stimdelay = 1 ;
stimdur = 2 ;
stim_amp = 35 ;

BCL_alt = zeros(variations,1) ;

numbertokeep = 2 ;

for ii=1:variations
  newparams = allparameters(:,ii) ;
  newparamscell = num2cell(newparams) ;
  [GNa_,GNab_,PCa_,GCab_,Gto_,GKr_,GKs_,GK1_,GpK_, ...
    INaK_,kNaCa,arel,crel,Vleak,Iup_,GpCa_] = deal(newparamscell{:}) ;
 
    
 % scaling = exp(sigma*randn(n_parameters,1)) ;
 % newparams = scaling.*baselineparameters ;
 % paramscell = num2cell(newparams) ;
 % allparameters(:,ii) = newparams ;
 % [GNa_,GCa_,Gto_,GK_,GK1_,GNab_,GCab_,gNaK,gNCX_] = deal(paramscell{:}) ;
  
  %% Initialize variables for finding threshold
  threshold = 0 ;
  anyalternans = 0 ;
  BCL_sub = 100 + 10*rand ;
  BCL_sup = stimint_initial + 10*randn ;
  stiminterval = BCL_sup ;

  while (threshold == 0)
    tdices = zeros(3*numbertokeep,1) ;
    
    tend = 50*stiminterval ;    

    n_stimuli = ceil((tend-stimdelay)/stiminterval) ;
    stim_starts = stimdelay + stiminterval*(0:n_stimuli-1)  ;
    stim_ends = stim_starts + stimdur ;

    simints = 3*n_stimuli ;
    for i=1:n_stimuli
      intervals(3*i-2,:) = [stiminterval*(i-1),stim_starts(i)] ;
      intervals(3*i-1,:) = [stim_starts(i),stim_ends(i)] ;
      intervals(3*i,:) = [stim_ends(i),stiminterval*i] ;
    end
    intervals(end,:) = [stim_ends(end),tend] ;

    Istim = zeros(simints,1) ;
    stimindices = 3*(1:n_stimuli) - 1 ;
    Istim(stimindices) = -stim_amp ;
    
V = -85 ;
m = 0 ;
h = 1 ;
j = 1 ;
d = 0 ;
f = 1 ;
fCa = 1 ;
r = 0 ;
s = 1 ;
xs = 0 ;
xr1 = 1 ;
xr2 = 1 ;
g = 0 ;
Cai = 0.1 ;
CaSR = 1000 ;
Nai = 10000 ;
Ki = 130000 ;

statevar_i = [V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] ;

    [post,posstatevars] = ode15s(@dydt_tentusscher,[0,30000],statevar_i,[],0) ;
    
    statevar_i = posstatevars(end,:) ;
  
    for i=1:simints-3*numbertokeep
      [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;
      
      statevar_i = posstatevars(end,:) ;
      t = post(end) ;
    end % for
    statevars = statevar_i ;
    tdicesdex = 1 ;
    for i=simints-3*numbertokeep+1:simints
      [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;
      
      t = [t;post(2:end)] ;
      tdices(tdicesdex) = length(t) ;
      tdicesdex = tdicesdex + 1 ;
      statevars = [statevars;posstatevars(2:end,:)] ;
      statevar_i = posstatevars(end,:) ;
    end % for
      
    t = t - min(t) ;

    outputcell = num2cell(statevars,1) ;
    nvariables = length(outputcell) ;

    [V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] = deal(outputcell{:}) ;

%     secondlast = V(tdices(1):tdices(3)) ;
%     secondlasttime = t(tdices(1):tdices(3)) - t(tdices(1)) ;
    secondlast = V(1:tdices(3)) ;
    secondlasttime = t(1:tdices(3)) - t(1) ;
    lastap = V(tdices(4):tdices(6)) ;
    lasttime = t(tdices(4):tdices(6)) - t(tdices(4)) ; ;

    % Vderiv = ([lastap(2:end);lastap(end)] - lastap) ;
    Vderiv = diff(lastap) ;
    [pos,dexmax] = max(Vderiv) ;
    tinit = lasttime(dexmax) ;
    [peakV,peakdex] = max(lastap) ;
    tpeak = lasttime(peakdex) ;
    lastrepoldex = find(lasttime > tpeak & lastap < -60) ;
    if (lastrepoldex)
      lastrepoltime = lasttime(lastrepoldex(1))-tinit ;
    end

    % Vderiv = ([secondlast(2:end);secondlast(end)] - secondlast) ;
    Vderiv = diff(secondlast) ;
    [pos,dexmax] = max(Vderiv) ;
    tinit = secondlasttime(dexmax) ;
    [peakV,peakdex] = max(secondlast) ;
    tpeak = secondlasttime(peakdex) ;
    secondlastrepoldex = find(secondlasttime > tpeak & secondlast < -60) ;
    if (secondlastrepoldex)
      secondlastrepoltime = secondlasttime(secondlastrepoldex(1))-tinit ;
    end

%     figure
%     hold on 
%     plot(secondlasttime,secondlast,'b')
%     plot(lasttime,lastap,'r')
    
    if (lastrepoltime & secondlastrepoltime)
      alternans = abs(lastrepoltime-secondlastrepoltime) ;
      if (alternans > 10) 
%         disp('Alternans')
        BCL_sub = stiminterval ;
%         anyalternans = 1 ;
        if ((BCL_sup-BCL_sub) < 5)
          threshold = BCL_sub ;
          BCL_alt(ii) = BCL_sub ;
        else
          stiminterval = 0.5*(BCL_sub + BCL_sup) ;          
        end
      else
%         disp('No alternans')
        BCL_sup = stiminterval ;
        stiminterval = 0.5*(BCL_sub + BCL_sup) ;          
      end
    else
%       disp('No alternans')
      BCL_sub = stiminterval ;
%       anyalternans = 1 ;
      if ((BCL_sup-BCL_sub) < 5)
        threshold = BCL_sub ;
        BCL_alt(ii) = BCL_sub ;
      else
        stiminterval = 0.5*(BCL_sub + BCL_sup) ;          
      end
    end
      
  end % while loop to determine alternans threshold 
      
      
end
  
BCL_alt;

% figure (1)
% plot(t,V)

save TNNP_alternans_fixed allparameters BCL_alt variations stimdur stimdelay n_stimuli

