% Ten Tusscher et al model
% Am J Physiol 286: H1573H1589, 2004.
% human ventricular cell 
% improved version of Priebe-Beucklemann
%    
%    t   time variable
%    V   membrane potantial
clear all
load tt_fixed_params
colors = repmat('krgbmc',1,500) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 1:  Define all constants
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global F R T RTF ;
% Physical constants
F = 96.4867 ;                % Faraday constant, coulombs/mmol
R = 8.314 ;                  % gas constant, J/(K mol)
T = 273+37 ;                 % absolute temperature, K 
RTF = R*T/F ;

global Cm S ;
Cm = 1 ;                      % uF/cm2
S = 0.2 ;                     % surface area to volume ratio

global Vmyo VSR ;
Vmyo = 16.404 ;               % pL
VSR = 1.094 ;                 % pL

global Ko Nao Cao ;
Ko = 5400 ;                   % uM
Nao = 140000 ;                % uM
Cao = 2000 ;                  % uM

global GNa_ PCa_ GK1_ GKr_ GKs_ Gto_ ;
global GpK_ GpCa_ GNab_ GCab_ pKNa ;
GNa_ = 14.838 ;               % nS/pF
PCa_ = 1.75e-4 ;              % cm3 uF-1 s-1 
GK1_ = 5.405 ;                % nS/pF
GKr_ = 0.096 ;                % nS/pF

GpK_ = 0.0146 ;               % nS/pF
GpCa_ = 0.025 ;               % nS/pF
GNab_ = 2.9e-4 ;              % nS/pF
GCab_ = 5.92e-4 ;             % nS/pF

% % % Epicardial cell
Gto_ = 0.294 ;                % nS/pF
GKs_ = 0.245 ;                % nS/pF

% % % % M cell
% Gto_ = 0.294 ;                % nS/pF
% GKs_ = 0.062 ;                % nS/pF
% 
% % % % Endocardial cell
% Gto_ = 0.073 ;                % nS/pF
% GKs_ = 0.245 ;                % nS/pF
% 

% % % % Epicardial cell, M cell
% Gto_ = 0.294 ;                 % nS/pF
% % % % Endocardial cell
% Gto_ = 0.073 ;                 % nS/pF
% 
% % % % Epicardial cell, endocardial cell
% GKs_ = 0.245 ;                 % nS/pF
% % % % M cell
% GKs_ = 0.062 ;                 % nS/pF

pKNa = 0.03 ;                  % relative permeability, Na to K

global INaK_ kNaCa Iup_ Vleak
% Maximum rates of intracellular transport mechanisms
INaK_ = 1.362 ;                % pA/pF
kNaCa = 1000 ;                 % pA/pF
Iup_ = 0.425 ;                 % uM/ms
Vleak = 8e-5 ;                 % ms-1

global KmNa KmCa ksat alpha_ncx eta KmNai KmKo KpCa Kmup ;
% global eta ksat KmNa KmCa KmfCa KmK1 KmNai KmKo KmpCa Kmup  
% global KmNa KmCa KmNai KmKo sigma ;
% % values needed for calculation of NCX current
% Several saturation constants, nomenclature gets tricky
% NCX
KmNa = 87500 ;                 % uM
KmCa = 1380 ;                  % uM
ksat = 0.1 ;                   % unitless
alpha_ncx = 2.5 ;              % unitless
eta = 0.35 ;                   % unitless, actually gamma in paper
% Na-K pump
KmNai = 40000 ;                % uM
KmKo = 1000 ;                  % uM
% Sarcolemmal Ca pump
KpCa = 0.5 ;                    % uM
% SERCA
Kmup = 0.25 ;                  % uM

global arel brel crel ;
arel = 16.464 ;                 % uM/ms
brel = 250 ;                   % uM
crel = 8.232 ;                  % uM/ms

% % % Buffering parameters
% % % Considered generic cytotolic and SR buffers
global Bufc Kbufc BufSR KbufSR 
Bufc = 150 ;                % uM
Kbufc = 1 ;                   % uM
BufSR = 10000 ;              % uM
KbufSR = 300 ;                 % uM 

baselineparameters = [GNa_;GNab_;PCa_;GCab_;Gto_;GKr_;GKs_;GK1_;GpK_; ...
  INaK_;kNaCa;arel;crel;Vleak;Iup_;GpCa_] ;

n_parameters = length(baselineparameters) ;
variations = 300 ;
% variations = 1 ;
sigma = 0.1 ;     % standard deviation of variation of parameters
% sigma = 0 ;




 
%  stimints = 370:-5:100 ;
%  %stimints = 400 ;
%  pacingrates = length(stimints) ;
%  allparameters = zeros(n_parameters,variations) ;

BCLs = [1000,400] ;
actionpotentials = cell(2*variations,length(BCLs)) ;
alternans = zeros(variations,length(BCLs)) ;

% figure(1)
% clf
% hold on
% 
goodcount = 1 ;
hcgcount = 1 ;

for ii=1:variations

  newparams = allparameters(:,ii) ;
  newparamscell = num2cell(newparams) ;
[GNa_,GNab_,PCa_,GCab_,Gto_,GKr_,GKs_,GK1_,GpK_, ...
    INaK_,kNaCa,arel,crel,Vleak,Iup_,GpCa_] = deal(newparamscell{:}) ;
 
  for iiii=1:length(BCLs)


    stiminterval = BCLs(iiii) ;      % interval bewteen stimuli, ms
    tend = 10*stiminterval ;              % end of simulation, ms
    stimdelay = 40 ;
    stimdur = 2 ;
    stim_amp = 30 ;

    n_stimuli = ceil((tend-stimdelay)/stiminterval) ;
    stim_starts = stimdelay + stiminterval*(0:n_stimuli-1)  ;
    stim_ends = stim_starts + stimdur ;

    simints = 3*n_stimuli ;
    for i=1:n_stimuli
      intervals(3*i-2,:) = [stiminterval*(i-1),stim_starts(i)] ;
      intervals(3*i-1,:) = [stim_starts(i),stim_ends(i)] ;
      intervals(3*i,:) = [stim_ends(i),stiminterval*i] ;
     
    end
    intervals(end,:) = [stim_ends(end),tend] ;

    Istim = zeros(simints,1) ;
    stimindices = 3*(1:n_stimuli) - 1 ;
    Istim(stimindices) = -stim_amp ;
    

    numbertokeep = 2 ;
    tdices = zeros(3*numbertokeep,1) ;
    tdices_all = zeros(3*numbertokeep,variations) ;

V = -85 ;
m = 0 ;
h = 1 ;
j = 1 ;
d = 0 ;
f = 1 ;
fCa = 1 ;
r = 0 ;
s = 1 ;
xs = 0 ;
xr1 = 1 ;
xr2 = 1 ;
g = 0 ;
Cai = 0.1 ;
CaSR = 1000 ;
Nai = 10000 ;
Ki = 130000 ;

statevar_i = [V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki]' ;

      [post,posstatevars] = ode15s(@dydt_tentusscher,[0,60000],statevar_i,[],0) ;
      statevar_i = posstatevars(end,:) ;
  
      if (n_stimuli > numbertokeep)
        for i=1:simints-3*numbertokeep
          [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;  
          statevar_i = posstatevars(end,:) ;
          t = post(end) ;
        end % for
        statevars = statevar_i ;
        tdicesdex = 1 ;
        for i=simints-3*numbertokeep+1:simints
          [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;
          t = [t;post(2:end)] ;
          tdices(tdicesdex) = length(t) ;
          tdicesdex = tdicesdex + 1 ;
          statevars = [statevars;posstatevars(2:end,:)] ;
          statevar_i = posstatevars(end,:) ;
        end % for
      else
        t = 0 ;
        statevars = statevar_i ;
        tdicesdex = 1 ;
        for i=1:simints
          [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;  

          t = [t;post(2:end)] ;
          tdices(tdicesdex) = length(t) ;
          tdicesdex = tdicesdex + 1 ;
          statevars = [statevars;posstatevars(2:end,:)] ;
          statevar_i = posstatevars(end,:) ;
        end % for
      end % if

    t = t - min(t) ;

    outputcell = num2cell(statevars,1) ;
    nvariables = length(outputcell) ;

    [V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] = deal(outputcell{:}) ;

%     figure(1)
%     hold on
%     plot(t,V,colors(mod(ii,length(colors))+1))


      secondlast = V(tdices(end-5):tdices(end-3)) ;
      secondlasttime = t(tdices(end-5):tdices(end-3)) - t(tdices(end-5)) ;
      lastap = V(tdices(end-2):tdices(end)) ;
      lasttime = t(tdices(end-2):tdices(end)) - t(tdices(end-2)) ; ;
    %   secondlast = V(tdices(1):tdices(3)) ;
    %   secondlasttime = t(tdices(1):tdices(3)) - t(tdices(1)) ;
    %   lastap = V(tdices(4):tdices(6)) ;
    %   lasttime = t(tdices(4):tdices(6)) - t(tdices(4)) ; ;


    if (lastap(end) > -70 | secondlast(end) > -70)

    else
      Vderiv = ([lastap(2:end);lastap(end)] - lastap) ;
      [pos,dexmax] = max(Vderiv) ;
      tinit = lasttime(dexmax) ;
      [peakV,peakdex] = max(lastap) ;
      tpeak = lasttime(peakdex) ;
      repoldex = find(lasttime > tpeak & lastap < -60) ;
      lastrepoltime = lasttime(repoldex(1))-tinit ;

      Vderiv = ([secondlast(2:end);secondlast(end)] - secondlast) ;
      [pos,dexmax] = max(Vderiv) ;
      tinit = secondlasttime(dexmax) ;
      [peakV,peakdex] = max(secondlast) ;
      tpeak = secondlasttime(peakdex) ;
      repoldex = find(secondlasttime > tpeak & secondlast < -60) ;
      secondlastrepoltime = secondlasttime(repoldex(1))-tinit ;

      alternans(ii,iiii) = abs(lastrepoltime-secondlastrepoltime) ;

      if (alternans(ii,iiii) > 20)

      else
        actionpotentials{2*ii-1,iiii} = lasttime ;
        actionpotentials{2*ii,iiii} = lastap ;
        allparameters(:,ii) = newparams ;
%         tdices_all(:,goodcount) = tdices ;
%         goodcount = goodcount + 1 ;
      end
    end
    figure(iiii)
    hold on 
    plot(lasttime,lastap,colors(mod(ii,length(colors))+1))

end

if (mod(ii,50) == 0)
  disp(['Completed ',int2str(ii),' iterations'])
end

end


% goodcount = goodcount - 1 ;
% hcgcount = hcgcount - 1 ;

% allparameters = allparameters(1:n_Gs,:) ;
% save bernusaps allparameters actionpotentials variations goodcount n_parameters n_Gs n_ps n_Vs 
% if (hcgcount)
%   save hcgaps hcgparams hcgaps variations hcgcount n_parameters 
% end
save frequency_fixed allparameters actionpotentials variations n_parameters 

